@@ -3,7 +3,7 @@
 ##' Simulate data from a general SEM model including non-linear effects and
 ##' general link and distribution of variables.
 ##'
-##' @aliases sim sim.lvmfit sim.lvm
+##' @aliases sim.lvmfit sim.lvm
 ##' simulate.lvmfit simulate.lvm
 ##' transform<- transform<-.lvm transform.lvm
 ##' functional functional<-  functional.lvm functional<-.lvm
@@ -35,6 +35,7 @@
 ##' threshold.lvm
 ##' ones.lvm Binary.lvm binary.lvm
 ##' Sequence.lvm
+##' none.lvm constant.lvm id.lvm
 ##' @usage
 ##' \method{sim}{lvm}(x, n = NULL, p = NULL, normal = FALSE, cond = FALSE,
 ##' sigma = 1, rho = 0.5, X = NULL, unlink=FALSE, latent=TRUE,
@@ -66,12 +67,12 @@
 ##' distribution(m,~y+z) <- binomial.lvm("logit")
 ##' d <- sim(m,1e3)
 ##' head(d)
-
+##'
 ##' e <- estimate(m,d,estimator="glm")
 ##' e
 ##' ## Simulate a few observation from estimated model
 ##' sim(e,n=5)
-
+##'
 ##' ##################################################
 ##' ## Poisson
 ##' ##################################################
@@ -80,9 +81,9 @@
 ##' head(d)
 ##' estimate(m,d,estimator="glm")
 ##' mean(d$z); lava:::expit(1)
-
+##'
 ##' summary(lm(y~x,sim(lvm(y[1:2]~4*x),1e3)))
-
+##'
 ##' ##################################################
 ##' ### Gamma distribution
 ##' ##################################################
@@ -92,11 +93,11 @@
 ##' d <- sim(m,1e4)
 ##' summary(g <- glm(y~x,family=Gamma(),data=d))
 ##' \dontrun{MASS::gamma.shape(g)}
-
+##'
 ##' args(lava::Gamma.lvm)
 ##' distribution(m,~y) <- Gamma.lvm(shape=2,log=TRUE)
 ##' sim(m,10,p=c(y=0.5))[,"y"]
-
+##'
 ##' ##################################################
 ##' ### Beta
 ##' ##################################################
@@ -105,7 +106,7 @@
 ##' var(sim(m,100,"y,y"=2))
 ##' distribution(m,~y) <- beta.lvm(alpha=2,beta=1,scale=FALSE)
 ##' var(sim(m,100))
-
+##'
 ##' ##################################################
 ##' ### Transform
 ##' ##################################################
@@ -114,7 +115,7 @@
 ##' regression(m) <- y~x+z+xz
 ##' d <- sim(m,1e3)
 ##' summary(lm(y~x+z + x*I(z>0),d))
-
+##'
 ##' ##################################################
 ##' ### Non-random variables
 ##' ##################################################
@@ -124,7 +125,7 @@
 ##'                                Binary.lvm(0.5),    ##  0.5n 0, 0.5n 1
 ##'                                Binary.lvm(interval=list(c(0.3,0.5),c(0.8,1))))
 ##' sim(m,10)
-
+##'
 ##' ##################################################
 ##' ### Cox model
 ##' ### piecewise constant hazard
@@ -133,7 +134,7 @@
 ##' rates <- c(1,0.5); cuts <- c(0,5)
 ##' ## Constant rate: 1 in [0,5), 0.5 in [5,Inf)
 ##' distribution(m,~t) <- coxExponential.lvm(rate=rates,timecut=cuts)
-
+##'
 ##' \dontrun{
 ##'     d <- sim(m,2e4,p=c("t~x"=0.1)); d$status <- TRUE
 ##'     plot(timereg::aalen(survival::Surv(t,status)~x,data=d,
@@ -143,7 +144,7 @@
 ##'                    method="linear")
 ##'     curve(L,0,100,add=TRUE,col="blue")
 ##' }
-
+##'
 ##' ##################################################
 ##' ### Cox model
 ##' ### piecewise constant hazard, gamma frailty
@@ -175,7 +176,7 @@
 ##' \dontrun{
 ##' mets::fast.reshape(sim(m,100),varying="t")
 ##' }
-
+##'
 ##' ##################################################
 ##' ### General multivariate distributions
 ##' ##################################################
@@ -183,17 +184,17 @@
 ##' m <- lvm()
 ##' distribution(m,~y1+y2,oratio=4) <- VGAM::rbiplackcop
 ##' ksmooth2(sim(m,1e4),rgl=FALSE,theta=-20,phi=25)
-
+##'
 ##' m <- lvm()
 ##' distribution(m,~z1+z2,"or1") <- VGAM::rbiplackcop
 ##' distribution(m,~y1+y2,"or2") <- VGAM::rbiplackcop
 ##' sim(m,10,p=c(or1=0.1,or2=4))
 ##' }
-
+##'
 ##' m <- lvm()
 ##' distribution(m,~y1+y2+y3,TRUE) <- function(n,...) rmvn0(n,sigma=diag(3)+1)
 ##' var(sim(m,100))
-
+##'
 ##' ## Syntax also useful for univariate generators, e.g.
 ##' m <- lvm(y~x+z)
 ##' distribution(m,~y,TRUE) <- function(n) rnorm(n,mean=1000)
@@ -201,7 +202,7 @@
 ##' distribution(m,~y,"m1",0) <- rnorm
 ##' sim(m,5)
 ##' sim(m,5,p=c(m1=100))
-
+##'
 ##' ##################################################
 ##' ### Regression design in other parameters
 ##' ##################################################
@@ -214,7 +215,7 @@
 ##' constrain(m,sd~x) <- function(x) exp(x)^.5
 ##' distribution(m,~y) <- function(n,mean,sd) rnorm(n,mean,sd)
 ##' if (interactive()) plot(y~x,sim(m,1e3))
-
+##'
 ##' ## Regression on variance parameter
 ##' m <- lvm()
 ##' regression(m) <- y~x
@@ -224,7 +225,7 @@
 ##' ## regression(m) <- v[NA:0]~x
 ##' distribution(m,~y) <- function(n,mean,v) rnorm(n,mean,exp(v)^.5)
 ##' if (interactive()) plot(y~x,sim(m,1e3))
-
+##'
 ##' ## Regression on shape parameter in Weibull model
 ##' m <- lvm()
 ##' regression(m) <- y ~ z+v
@@ -233,26 +234,26 @@
 ##' distribution(m,~cens) <- coxWeibull.lvm(scale=1)
 ##' distribution(m,~y) <- coxWeibull.lvm(scale=0.1,shape=~s)
 ##' eventTime(m) <- time ~ min(y=1,cens=0)
-
+##'
 ##' if (interactive()) {
 ##'     d <- sim(m,1e3)
 ##'     require(survival)
 ##'     (cc <- coxph(Surv(time,status)~v+strata(x,z),data=d))
 ##'     plot(survfit(cc) ,col=1:4,mark.time=FALSE)
 ##' }
-
+##'
 ##' ##################################################
 ##' ### Categorical predictor
 ##' ##################################################
 ##' m <- lvm()
 ##' ## categorical(m,K=3) <- "v"
 ##' categorical(m,labels=c("A","B","C")) <- "v"
-
+##'
 ##' regression(m,additive=FALSE) <- y~v
 ##' \dontrun{
 ##' plot(y~v,sim(m,1000,p=c("y~v:2"=3)))
 ##' }
-
+##'
 ##' m <- lvm()
 ##' categorical(m,labels=c("A","B","C"),p=c(0.5,0.3)) <- "v"
 ##' regression(m,additive=FALSE,beta=c(0,2,-1)) <- y~v
@@ -273,22 +274,6 @@
 ##' m2 <- sim(m,'y~x'=2)
 ##' sim(m,10,'y~x'=2)
 ##' sim(m2,10) ## Faster
-##'
-"sim" <- function(x,...) UseMethod("sim")
-
-##' @export
-sim.lvmfit <- function(x,n=nrow(model.frame(x)),p=pars(x),xfix=TRUE,...) {
-    m <- Model(x)
-    if ((nrow(model.frame(x))==n) & xfix) {
-        X <- exogenous(x)
-        mydata <- model.frame(x)
-        for (pred in X) {
-            distribution(m, pred) <- list(mydata[,pred])
-        }
-    }
-    sim(m,n=n,p=p,...)
-}
-
 ##' @export
 sim.lvm <- function(x,n=NULL,p=NULL,normal=FALSE,cond=FALSE,sigma=1,rho=.5,
             X=NULL,unlink=FALSE,latent=TRUE,use.labels=TRUE,seed=NULL,...) {
@@ -445,7 +430,7 @@ sim.lvm <- function(x,n=NULL,p=NULL,normal=FALSE,cond=FALSE,sigma=1,rho=.5,
                     if (is.function(dist.x)) {
                         res[,X.idx[i]] <- dist.x(n=n,mu=mu.x,var=P[X.idx[i],X.idx[i]])
                     } else {
-                        if (is.null(dist.x) || is.na(dist.x)) {
+                        if (is.null(dist.x) || any(is.na(dist.x))) {
                         } else {
                             if (length(dist.x)!=n) stop("'",vv[X.idx[i]], "' fixed at length ", length(dist.x)," != ",n,".")
                             res[,X.idx[i]] <- dist.x ## Deterministic
@@ -481,7 +466,7 @@ sim.lvm <- function(x,n=NULL,p=NULL,normal=FALSE,cond=FALSE,sigma=1,rho=.5,
         I <- diag(nrow=length(nn))
         IAi <- Inverse(I-t(A))
         colnames(E) <- vv
-        dd <- t(apply(heavytail.sim.hook(x,E),1,function(x) x+mu))
+        dd <- t(apply(heavytail_sim_hook(x,E),1,function(x) x+mu))
         res <- dd%*%t(IAi)
         colnames(res) <- vv
     } else {
@@ -503,8 +488,9 @@ sim.lvm <- function(x,n=NULL,p=NULL,normal=FALSE,cond=FALSE,sigma=1,rho=.5,
             return(x)
         }
 
-        if (length(xconstrain)>0)
-          for (i in which(xconstrain.idx)) {
+      if (length(xconstrain)>0) {
+        for (i in which(xconstrain.idx)) {
+          if (!(names(xconstrain.idx[i]) %in% nn)) { ## A parameter and not a variable
             ff <- constrain(x)[[i]]
             myargs <- attributes(ff)$args
             D <- matrix(0,n,length(myargs))
@@ -514,17 +500,19 @@ sim.lvm <- function(x,n=NULL,p=NULL,normal=FALSE,cond=FALSE,sigma=1,rho=.5,
               else
                 D[,j] <- M$parval[[myargs[j]]]
             }
-            val <- try(apply(D,1,ff),silent=TRUE)
-            if (inherits(val,"try-error") || NROW(val)<n) val <- ff(D)
+            val <- try(val <- ff(D), silent=TRUE)
+            if (inherits(val,"try-error") || NROW(val)<n) apply(D,1,ff)
             res <- cbind(res, val); colnames(res)[ncol(res)] <- names(xconstrain.idx)[i]
           }
+        }
+      }
 
         if (any(xconstrain.par%in%covparnames)) {
             mu0 <- rep(0,ncol(P))
             P0 <- P
             E <- t(sapply(seq_len(n),function(idx) {
-                for (i in intersect(xconstrain.par,covparnames)) {
-                    P0[covariance(x)$labels==i] <- res[idx,i]
+              for (i in intersect(xconstrain.par,covparnames)) {
+                  P0[covariance(x)$labels==i] <- res[idx,i]
                 }
                 PP <- with(svd(P0), v%*%diag(sqrt(d),nrow=length(d))%*%t(u))
                 return(mu0+rbind(rnorm(ncol(P0)))%*%PP)
@@ -532,7 +520,7 @@ sim.lvm <- function(x,n=NULL,p=NULL,normal=FALSE,cond=FALSE,sigma=1,rho=.5,
         }
 
         colnames(E) <- vv
-        E <- heavytail.sim.hook(x,E)
+        E <- heavytail_sim_hook(x,E)
 
         ## Non-linear regression components
         xconstrain <- c()
@@ -550,9 +538,8 @@ sim.lvm <- function(x,n=NULL,p=NULL,normal=FALSE,cond=FALSE,sigma=1,rho=.5,
                 xconstrain <- c(xconstrain,list(el))
             }
         }
-
         yconstrain <- unlist(lapply(xconstrain,function(x) x$endo))
-        for (i in exo_constrainY) {
+      for (i in exo_constrainY) {
             cc <- x$constrainY[[i]]
             args <- cc$args
             args <- if (is.null(args) || length(args)==0) res[,i] else res[,args]
@@ -657,7 +644,7 @@ sim.lvm <- function(x,n=NULL,p=NULL,normal=FALSE,cond=FALSE,sigma=1,rho=.5,
                                                     res[,exo,drop=FALSE])
                                           else res[,exo,drop=FALSE])
                                 yy <- try(with(xconstrain[[ipos]],
-                                              func(X[,myidx])),silent=TRUE)
+                                               func(X[,myidx,drop=FALSE])),silent=TRUE)
                                 if (NROW(yy) != NROW(res)) { ## apply row-wise
                                     mu.i <- #mu.i +
                                         with(xconstrain[[ipos]],
@@ -726,17 +713,19 @@ sim.lvm <- function(x,n=NULL,p=NULL,normal=FALSE,cond=FALSE,sigma=1,rho=.5,
                             if (length(x$constrainY)>0 && i%in%names(x$constrainY)) {
                                 cc <- x$constrainY[[i]]
                                 args <- cc$args
-                                args <- if (is.null(args) || length(args)==0)
+                                args <- if (length(args)==0)
                                            res[,pos]
                                        else {
                                            ii <- intersect(names(M$parval),args)
                                            args0 <- args
-                                           args <- res[,intersect(args,colnames(res)),drop=FALSE]
+                                           args <- res[,intersect(args0,colnames(res)),drop=FALSE]
+
                                            if (length(ii)>0) {
                                                pp <- rbind(unlist(M$parval[ii]))%x%cbind(rep(1,n))
                                                colnames(pp) <- ii
                                                args <- cbind(res,pp)[,args0,drop=FALSE]
                                            }
+                                           args
                                        }
                                 res[,pos] <- cc$fun(args,p) # ,...)
                             }
@@ -765,15 +754,25 @@ sim.lvm <- function(x,n=NULL,p=NULL,normal=FALSE,cond=FALSE,sigma=1,rho=.5,
     return(res)
 }
 
-
-
 ##' @export
-simulate.lvm <- function(object,nsim,seed=NULL,...) {
-    sim(object,nsim,seed=seed,...)
+sim.lvmfit <- function(x, n = nrow(model.frame(x)), p = pars(x), xfix = TRUE, ...) {
+  m <- Model(x)
+  if ((nrow(model.frame(x)) == n) & xfix) {
+    X <- exogenous(x)
+    mydata <- model.frame(x)
+    for (pred in X) {
+      distribution(m, pred) <- list(mydata[, pred])
+    }
+  }
+  sim(m, n = n, p = p, ...)
 }
 
 ##' @export
-simulate.lvmfit <- function(object,nsim,seed=NULL,...) {
-    sim(object,nsim,seed=seed,...)
+simulate.lvm <- function(object, nsim, seed = NULL, ...) {
+  sim(object, nsim, seed = seed, ...)
 }
 
+##' @export
+simulate.lvmfit <- function(object, nsim, seed = NULL, ...) {
+  sim(object, nsim, seed = seed, ...)
+}
