@@ -5,7 +5,7 @@ variances <- function(x,mean=FALSE) {
 ##  if (is.null(x$parpos))
 ##    x$parpos <- parpos(x)
   x$parpos <- parpos(Model(x),mean=TRUE)
-  res <- diag(x$parpos$P)[diag(index(x)$P0)==1]
+  res <- diag(x$parpos$P)[which(diag(index(x)$P0)==1)]
   if (!mean) {
     return(res - index(x)$npar.mean)
   }
